/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Clippings.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Integration with main window and message view window
//


var gClippings = {
  dataSrcInitialized:  false,
  showDialog:          true,
  appUtils:            null,
  clippingsSvc:        null,
  strBundle:           null,

  statusBarObserver: {
    onDragOver: function (aEvent, aFlavour, aSession)
    {
      aEvent.target.setAttribute("ae-dnd", "true");
    },

    onDragExit: function (aEvent, aSession)
    {
      aEvent.target.removeAttribute("ae-dnd");
    },

    onDrop: function (aEvent, aTransferData, aSession)
    {
      aEvent.target.removeAttribute("ae-dnd");

      if (! gClippings.dataSrcInitialized) {
	// The RDF data source has to be initialized if it has not already been
	// done so, otherwise RDF node creation will fail, and the new
	// Clippings entry will never be created.
	// This initialization is done in the code for the Clippings popup
	// menu's `onpopupshowing' event handler.
	gClippings.initClippingsPopup(document.getElementById("ae-clippings-popup-1"), document.getElementById("ae-clippings-menu-1"));
      }

      var text = aTransferData.data;
      var result = gClippings.clippingsSvc.newClippingFromText(text, gClippings.showDialog, window, null, false);
      if (result) {
	window.setTimeout(function () { gClippings.saveClippings() }, 100);
      }
    },

    getSupportedFlavours: function ()
    {
      var flavours = new FlavourSet();
      flavours.appendFlavour("text/unicode");
      return flavours;
    }
  },


  // Method handleEvent() effectively makes gClippings an implementation of
  // the EventListener interface; therefore it can be passed as the listener
  // argument to window.addEventListener() and window.removeEventListener()

  handleEvent: function (aEvent)
  {
    // When this method is invoked, 'this' will not refer to gClippings
    var _this = gClippings;

    if (aEvent.type == "load") {
      _this.initClippings();
    }
    else if (aEvent.type == "unload") {
      window.removeEventListener("load", _this, false);
      window.removeEventListener("unload", _this, false);
    }
  },


  //
  // Methods invoked by overlay code
  //

  alert: function (aMessage)
  {
    var title = this.strBundle.getString('appName');
    this.appUtils.alertEx(title, aMessage);
  },


  newFromClipboard: function () 
  {
    if (! this.dataSrcInitialized) {
      this.alert("ERROR: Clippings hasn't been initialized yet!");
      return;
    }

    var txt = this.appUtils.getTextFromClipboard();
    if (! txt) {
      this.alert(this.strBundle.getString('errorNoClipboardContent'));
      return;
    }

    var result = this.clippingsSvc.newClippingFromText(txt, this.showDialog, window, null, false);
    if (result) {
      window.setTimeout(function () { gClippings.saveClippings() }, 1);
    }
  },


  getSelectedText: function ()
  {
    var rv;
    var focusedWnd = document.commandDispatcher.focusedWindow;
    rv = focusedWnd.getSelection().toString();
    return rv;
  },


  openClippingsManager: function () 
  {
    var wnd = window.open("chrome://clippings/content/clippings.xul",
			  "clippings_wndobj", "chrome,resizable");
    wnd.focus();
  },


  insertClippingText: function (aText) 
  {
    this.appUtils.copyTextToClipboard(aText);

    try {
      // Paste clipping.  The following function is defined in
      // "chrome://global/content/globalOverlay.js"
      goDoCommand("cmd_paste");
      // SIDE EFFECT: The clipping text will remain on the system clipboard.
    }
    catch (e) {
      // Exception thrown if command is disabled or not applicable
      this.appUtils.beep();
    }
  },


  saveClippings: function () 
  {
    var title = this.strBundle.getString('appName');
    try {
      this.clippingsSvc.flushDataSrc();
    }
    catch (e if e.result == Components.results.NS_ERROR_NOT_INITIALIZED) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorSaveFailedDSNotInitialized"));
      return;
	   }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorOutOfMemory"));
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      var msg = aeStringUtils.format("%s: %s",
			      this.strBundle.getString("errorAccessDenied"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      var msg = aeStringUtils.format("%s: %s",
			      this.strBundle.getString("errorFileLocked"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      var msg = aeStringUtils.format("%s: %s",
			      this.strBundle.getString("errorFileTooBig"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_READ_ONLY) {
      var msg = aeStringUtils.format("%s: %s",
			      this.strBundle.getString('errorFileReadOnly'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_DISK_FULL) {
      var msg = aeStringUtils.format("%s: %s",
			      this.strBundle.getString('errorDiskFull'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e) {
      this.appUtils.alertEx(title, this.strBundle.getString("alertSaveFailed"));
      return;
    }
  },


  //
  // Browser window and Clippings menu initialization
  //

  isClippingsInitialized: false,

  initClippings: function ()
  {  
    // Workaround to this init function being called multiple times.
    if (this.isClippingsInitialized) {
      return;
    }

    this.appUtils = new aeClippingsUtils();
    this.strBundle = document.getElementById("ae-clippings-strings");

    try {
      this.clippingsSvc = Components.classes["clippings@mozdev.org/clippings;1"].getService(Components.interfaces.nsIClippingsService);
    }
    catch (e) {
      this.alert(e);
    }

    // String used for empty (null) clippings
    this.clippingsSvc.setEmptyClippingString(this.strBundle.getString('emptyClippingLabel'));

    if (this.appUtils.getPref("clippings.first_run") == true) {
      this.appUtils.setPref("clippings.first_run", false);
    }

    // First-run initialization after upgrade from 2.x -> 3.0
    if (this.appUtils.getPref("clippings.v3.first_run") == true) {
      var clippings3 = new aeClippings3(this.clippingsSvc, this.strBundle);
      var initFinished = clippings3.startInit();
      if (initFinished) {
	this.appUtils.setPref("clippings.v3.first_run", false);
      }
    }

    if (this.appUtils.PORTABLE_APP_BUILD && this.appUtils.isCommonDataSourceEnabled()) {
      this.appUtils.log("gClippings.initClippings(): Forcibly disabling common clippings on Portable Thunderbird");
      this.appUtils.setPref("clippings.datasource.common", false);
    }

    // Initialize the datasource in the Clippings XPCOM service
    var err = false;
    var dsURL = this.appUtils.getDataSourcePathURL() + this.appUtils.CLIPDAT_FILE_NAME;
    try {
      var ds = this.clippingsSvc.getDataSource(dsURL);
    }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      err = this.strBundle.getString("errorOutOfMemory");
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      err = aeStringUtils.format("%s: %s",
			  this.strBundle.getString("errorAccessDenied"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      err = aeStringUtils.format("%s: %s",
			  this.strBundle.getString("errorFileLocked"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      err = aeStringUtils.format("%s: %s",
			  this.strBundle.getString("errorFileTooBig"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e) {
      // File is corrupt - open Clippings Manager and perform recovery.
      err = true;
    }

    if (err) {
      this.openClippingsManager();
      return;
    }

    // Clippings backup
    var backupDirURL = this.appUtils.getDataSourcePathURL() + this.appUtils.BACKUP_DIR_NAME;
    this.clippingsSvc.setBackupDir(backupDirURL);
    this.clippingsSvc.setMaxBackupFiles(this.appUtils.getPref('clippings.backup.maxfiles'));

    this.dataSrcInitialized = true;

    var commonDSVerifyMsg = "";
    if (this.appUtils.getPref("clippings.datasource.common")) {
      commonDSVerifyMsg = "Enabled common data source between Firefox and Thunderbird.";
    }

    this.appUtils.log(aeStringUtils.format("gClippings.initClippings(): Clippings data source successfully loaded.\nHost app: %s (version %s)\nInitializing Clippings integration with host app window:\n%s\n%s", this.appUtils.getHostAppName(), this.appUtils.getHostAppVersionStr(), window.location.href, commonDSVerifyMsg));

    // Add null clipping to root folder if there are no items
    if (this.appUtils.getPref("clippings.datasource.process_root") == true) {
      this.clippingsSvc.processRootFolder();
      this.appUtils.setPref("clippings.datasource.process_root", false);
    }

    this.showDialog = !(this.appUtils.getPref("clippings.entries.add_silently"));

    // Place the status bar icon so that it appears as the last item, before
    // the window resizer grippy
    var statusBar = document.getElementById("status-bar");
    var statusBarPanel = document.getElementById("ae-clippings-icon");
    statusBar.appendChild(statusBarPanel);

    // Dynamically create status bar icon popup menu.  Defining it in
    // tbMessengerOverlay.xul won't work - it seems to disappear during loading
    // of the Messenger window.
    var popup = document.createElement("menupopup");
    popup.id = "ae-clippings-popup";

    var openClippingsMgr = document.createElement("menuitem");
    openClippingsMgr.setAttribute("label", this.strBundle.getString("openClippingsMgr"));
    openClippingsMgr.setAttribute("command", "ae_clippings_manager");
    openClippingsMgr.setAttribute("default", "true");

    var newFromClpbd = document.createElement("menuitem");
    newFromClpbd.setAttribute("command", "ae_new_clipping_from_clpbd");
    
    popup.appendChild(openClippingsMgr);
    popup.appendChild(newFromClpbd);
    document.getElementById("messengerWindow").appendChild(popup);

    // Initialize "New From Clipboard" command on status bar icon menu.
    var ellipsis = this.showDialog ? this.strBundle.getString("ellipsis") : "";
    var newFromClpbdCmd = document.getElementById("ae_new_clipping_from_clpbd");
    newFromClpbdCmd.setAttribute("label",
				 this.strBundle.getString("newFromClipbd")
				 + ellipsis);

    this.isClippingsInitialized = true;
  }
};


//
// Event handler initialization
//

window.addEventListener("load", gClippings, false);
window.addEventListener("unload", gClippings, false);
